/*
 * findslot.c
 * 
 * Kevin Smith 03/23/88
 * 
 */
#include "copyright2.h"

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <pwd.h>
#include <string.h>
#include <ctype.h>
#ifdef hpux
#include <time.h>
#else                           /* hpux */
#include <sys/time.h>
#endif
#include "netrek.h"

#ifdef EM
#define WAITMOTD
#endif

#define WAITWIDTH 180
#define WAITHEIGHT 60
#define WAITTITLE 15		/* height of title for wait window */
#define WAITICONHEIGHT 50
#define WAITICONWIDTH  50

extern int	newMotdStuff;	/* from newwwin.c */

#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* findslot.c */
static void mapWaitCount P_((W_Window countWin, unsigned int count));
static void mapMotdButtonWin P_((W_Window motdButtonWin));
static void mapWaitIcon P_((W_Window waitIcon, unsigned int count));

#undef P_

int
findslot()
{
   int             oldcount = -1;
   W_Window        waitWin, qwin, countWin, motdButtonWin, waitIcon;
   W_Event         event;
   struct timeval  tv;
   fd_set          mask;

   /* Wait for some kind of indication about in/not in */
   while (queuePos == -1) {
      socketPause();
      if (isServerDead()) {
	 printf("Augh!  Ghostbusted!\n");
	 exit(0);
      }
      readFromServer(NULL);
      if (me != NULL) {
	 /* We are in! */
	 return (me->p_no);
      }
   }

   /* We have to wait.  Make appropriate windows, etc... */
   waitWin = W_MakeWindow("wait", 0, 0, WAITWIDTH, WAITHEIGHT, NULL, 2, foreColor);

   countWin = W_MakeWindow("count", WAITWIDTH / 3, WAITTITLE, WAITWIDTH / 3,
			   WAITHEIGHT - WAITTITLE - 1, waitWin, 1, foreColor);
   qwin = W_MakeWindow("waitquit", 0, WAITTITLE, WAITWIDTH / 3, 
			   WAITHEIGHT - WAITTITLE - 1, waitWin, 1, foreColor);
   motdButtonWin = W_MakeWindow("motd_select", 2 * WAITWIDTH / 3 -1, WAITTITLE,
      WAITWIDTH, WAITHEIGHT - WAITTITLE - 1, waitWin, 1, foreColor);
   waitIcon = W_MakeWindow("wait_icon", 0, 0, WAITICONWIDTH, WAITICONHEIGHT,
			   NULL, BORDER, foreColor);
   W_SetIconWindow(waitWin, waitIcon);

   W_MapWindow(waitWin);
   W_MapWindow(countWin);
   W_MapWindow(qwin);
   W_MapWindow(motdButtonWin);
   W_Flush();

   for (;;) {
      /* improve the response of the waitQ window -- don't wait
         1 second before doing anything */
      while (!W_EventsPending()) {
	 tv.tv_sec = 1;
	 tv.tv_usec = 0;
	 FD_ZERO(&mask);
	 FD_SET(W_Socket(), &mask);
	 FD_SET(sock, &mask);
	 select(32, &mask, 0, 0, &tv);

	 if(FD_ISSET(sock, &mask)){
	    readFromServer(NULL);
	    if (isServerDead()) {
	       printf("We've been ghostbusted!\n");
	       exit(0);
	    }
	    if(motdWin && newMotdStuff)
	       showMotd(motdWin, motdw_line);
	    if (queuePos != oldcount) {
	       mapWaitCount(countWin, queuePos);
	       mapWaitIcon(waitIcon, queuePos);
	       oldcount = queuePos;
	    }
	    if (me != NULL) {
	       if(motdWin)
		  W_UnmapWindow(motdWin);
	       /* xx: tvtwm may crash if destroy followed by map
		  too quickly */
	       W_UnmapWindow(waitWin);
	       W_UnmapWindow(waitIcon);
	       W_Beep();
	       W_Beep();
	       W_Beep();

	       W_Flush();
	       W_DestroyWindow(waitWin);
	       W_DestroyWindow(waitIcon);

	       return (me->p_no);
	    }
	 }
      }
      if(!W_EventsPending())
	 continue;

      W_NextEvent(&event);
      switch ((int) event.type) {

      case W_EV_KEY:
	 if (motdWin && event.Window == motdWin) {
	    motdWinEvent(event.key);
	 } else if (event.Window == motdButtonWin) {
	    showMotdWin();
	 }
	 break;

      case W_EV_BUTTON:
	 if (event.Window == qwin) {
	    printf("OK, bye!\n");
	    exit(0);
	 } else if (event.Window == motdButtonWin) {
	    showMotdWin();
	 } else if (event.Window == waitIcon) {
	    mapWaitIcon(waitIcon, queuePos);
	 }
	 break;

      case W_EV_EXPOSE:
	 if (event.Window == waitWin) {
	    mapWaitWin(waitWin);
	 } else if (event.Window == qwin) {
	    mapWaitQuit(qwin);
	 } else if (event.Window == countWin) {
	    mapWaitCount(countWin, queuePos);
	 } else if (motdWin && event.Window == motdWin) {
	    showMotd(motdWin, motdw_line);
	 } else if (event.Window == motdButtonWin) {
	    mapMotdButtonWin(motdButtonWin);
	 } else if (event.Window == waitIcon) {
	    mapWaitIcon(waitIcon, queuePos);
	 }
	 break;
      default:
	 break;
      }
   }
}

void
mapWaitWin(waitWin)
    W_Window        waitWin;
{
   char           *s = "Netrek:  Game is full.";

   W_WriteText(waitWin, 15, 5, textColor, s, strlen(s), W_RegularFont);
}

void
mapWaitQuit(qwin)
    W_Window        qwin;
{
   char           *s = "Quit";

   W_WriteText(qwin, 15, 15, textColor, s, strlen(s), W_RegularFont);
}

static void
mapWaitCount(countWin, count)
    W_Window        countWin;
    unsigned int    count;
{
   char           *s = "Wait";
   char           *t = "Queue";
   char            buf[10];
   register int    len;

   W_WriteText(countWin, 15, 5, textColor, s, strlen(s), W_RegularFont);
   W_WriteText(countWin, 20, 15, textColor, t, strlen(t), W_RegularFont);
   sprintf(buf, "%d  ", count);
   len = /* strlen(buf) */ 3;
   if (count == -1)
      strcpy(buf, "?");
   W_WriteText(countWin, WAITWIDTH / 6 - len * W_Textwidth / 2, 25,
	       textColor, buf, len, W_RegularFont);
}

static void
mapMotdButtonWin(motdButtonWin)
    W_Window        motdButtonWin;
{
   char           *s = "MOTD";

   W_WriteText(motdButtonWin, 15, 15, textColor, s, strlen(s), W_RegularFont);
}

static void
mapWaitIcon(waitIcon, count)
    W_Window        waitIcon;
    unsigned int    count;
{
   char            buf[5];
   int             len;

   sprintf(buf, "%d", count);
   len = strlen(buf);
   W_WriteText(waitIcon, WAITICONWIDTH / 2 - 10, W_Textheight, textColor,
	       buf, len,
	       W_BigFont);
}
